
function [EEG, command] = pop_read_mayo(); 
command = '';
EEG = eeg_emptyset;


% get mef directory
[filename, filepath] = uigetfile('*.MEF;*.mef', 'Choose any MEF file -- pop_read_mayo()'); 
drawnow;
if filepath == 0 
	return; 
end

% read EEG (mef) files
disp('Preparing...');
files = dir([filepath '*.mef']);
n_channels = length(files);
fp = fopen([filepath files(1).name], 'r');
fseek(fp, 16, 'bof');     % number of entries field
n_samples = fread(fp, 1, 'uint64', 0, 'b');
fseek(fp, 64, 'bof');     % sampling frequency field
samp_freq = fread(fp, 1, 'double', 0, 'b');
fclose(fp);

% get downsampled and high cut frequencies
ds_samp_freq = 2000;
high_cut = ds_samp_freq / 5;
geometry = { [1 0.5] [1 0.5] [1 0.5] [1 0.5] [1 0.5] };
uilist = { ...
	{ 'Style', 'text', 'string', 'Current Sample Frequency (Hz):', 'fontweight', 'bold' } { 'Style', 'text', 'string', num2str(samp_freq) } ...
	{ 'Style', 'text', 'string', 'Downsampled Frequency (Hz):', 'fontweight', 'bold' } { 'Style', 'edit', 'string', num2str(ds_samp_freq) } ...
	{ 'Style', 'text', 'string', 'Lowpass Filter Setting (Hz):', 'fontweight', 'bold' } { 'Style', 'edit', 'string', num2str(high_cut) } ...
    { 'Style', 'text', 'string', '(Recommend 1/5 of the downsampled', 'fontweight', 'bold' } {} ...
    { 'Style', 'text', 'string', 'frequency for the lowpass filter)', 'fontweight', 'bold' } {} ...
};
result = inputgui( geometry, uilist, [], 'Downsampled Frequency');
if length( result ) == 0 return; end;
ds_samp_freq = str2num(result{1});
high_cut = str2num(result{2});
if ds_samp_freq > samp_freq
    disp('Downsampled frequency exceeds sampling frequency - setting equal');
    ds_samp_freq = samp_freq;
end
if high_cut > (ds_samp_freq / 2)
    disp('Lowpass filter setting exceeds Nyquist frequency - setting equal');
    high_cut = (ds_samp_freq / 2);
end
disp(['Original Frequency: ' num2str(samp_freq)]);
disp(['Downsampled Freqeuncy: ' num2str(ds_samp_freq)]);
disp(['Lowpass Filter Setting: ' num2str(high_cut)]); 

% Set up filter
filt_win = high_cut / (ds_samp_freq / 2);
if filt_win == 1
    filt_win = 1 - eps;
end
[b,a] = butter(5, filt_win, 'low');  % filter parameters: 5th order, lowpass design

% get memory 
ds_samp_num = floor((n_samples / samp_freq) * ds_samp_freq);
last_ds_samp = ((samp_freq / ds_samp_freq) * (ds_samp_num - 1)) + 1;
ds_times = linspace(1, last_ds_samp, ds_samp_num);
data = zeros(n_channels, ds_samp_num, 'single');
cv_bytes = zeros(4, 1, 'int8');
seg_size = 40000000;    % change this if running into memory problems
if (seg_size > n_samples)
    seg_size = n_samples;
end
pack;

n_segs = ceil(n_samples / seg_size);
for i = 1:n_channels
    disp(['Working on channel file "' files(i).name '" (' num2str(i) ' of ' num2str(n_channels) ') ...']);
    fp = fopen([filepath files(i).name], 'r');
    fseek(fp, 24, 'bof');
    chan_name = fread(fp, 32, 'int8');
    j = 1;
    while (chan_name(j));  
        EEG.chanlocs(i).labels(j) = char(chan_name(j));
        j = j + 1;
    end
    j = 1;
    current_val = 0;
    comp_idx = 1;
    ds_samp_start = 1;
    for m = 1:n_segs
        disp(['    Processing segment ' num2str(m) ' of ' num2str(n_segs) '...']);
        seg_start = ((m - 1) * seg_size) + 1;
        if m == n_segs
            seg_end = n_samples;
        else
            seg_end = m * seg_size;
        end
        seg_len = (seg_end - seg_start) + 1;
        disp('        Decompressing...');
        int_channel_data = decomp_mayo([filepath files(i).name], seg_start, seg_end);
        channel_data = double(int_channel_data);
        clear int_channel_data;
        disp('        Filtering...');
        channel_data = filtfilt(b, a, channel_data);
        disp('        Downsampling...');
        ds_samp_end = ds_samp_start;
        while ds_samp_end <= ds_samp_num
            if ds_times(ds_samp_end) <= seg_end
                ds_samp_end = ds_samp_end + 1;
            else
                break;
            end
        end
        ds_samp_end = ds_samp_end - 1;
        for q = ds_samp_start:ds_samp_end   % linear interpolation
        	x = ds_times(q);
        	fx = floor(x);
        	dx = x - fx;
            fx = (fx - seg_start) + 1;
        	fy = channel_data(fx);
        	data(i, q) = single(((channel_data(fx + 1) - fy) * dx) + fy);
    	end
        ds_samp_start = ds_samp_end + 1;
        clear channel_data
    end
end

% read a time file
disp('Reading times...');
files = dir([filepath '*.mtf']);
fp = fopen([filepath files(1).name], 'r');
fseek(fp, 16, 'bof');     % number of time entries offset
n_times = fread(fp, 1, 'uint64', 0, 'b');
fseek(fp, 1024, 'bof');
time_arr_len = n_times * 3;
times = fread(fp, time_arr_len, 'uint64', 0, 'b');
fclose(fp);


% read the event file
disp('Reading events...');
files = dir([filepath '*.mvf']);
fp = fopen([filepath files(1).name], 'r');
fseek(fp, 16, 'bof');     % number of event entries offset
n_events = fread(fp, 1, 'uint64', 0, 'b');
fseek(fp, 1024, 'bof');

time_idx = 1;
n_port_events = 0;
time_arr_len = time_arr_len - 3;
for i = 1:n_events
    type = fread(fp, 4, 'int8', 0, 'b');
    size = fread(fp, 1, 'uint32', 0, 'b');
    if (type(1) == 'P' && type(2) == 'I' && type(3) == '1' && type(4) == 0)
        n_port_events = n_port_events + 1;
        ev_time = fread(fp, 1, 'uint64', 0, 'b');
        ev_val = fread(fp, 1, 'uint64', 0, 'b');
        % find sample number
        while (1)
            if (time_idx <= time_arr_len)
                if (times(time_idx + 3) <= ev_time)
                    time_idx = time_idx + 3;
                else
                    break;
                end
            else
                break;
            end
        end
        time_sample_idx = times(time_idx + 2);
        ev_sample_idx = time_sample_idx + (((ev_time - times(time_idx)) / 1000000) * samp_freq);
        events(n_port_events).type = int32(ev_val);
        events(n_port_events).latency = round((ev_sample_idx * ds_samp_freq) / samp_freq) + 1;
        fseek(fp, size - 24, 'cof');
    else
        fseek(fp, size - 8, 'cof');
    end
end
fclose(fp);


% create eeglab structures
EEG.filepath = filepath; 
EEG.data = data;
EEG.srate = ds_samp_freq;
EEG.setname = '';
EEG.pnts = ds_samp_num;
EEG.xmin = 0;
EEG.xmax = (ds_samp_num - 1) / ds_samp_freq;
EEG.nbchan = n_channels;
EEG.trials = 1;
EEG.event = events;
EEG.times = ds_times;
EEG = eeg_checkset(EEG,'eventconsistency');
EEG = eeg_checkset(EEG,'makeur');

command = 'EEG = pop_read_mayo();';

return;

